#!/bin/bash
#
# rename and move photos
# author timm@ganske.org 2020
#
# You may set
#
# T85exifsrcdir      -- Where the pics and movies are before processing (if not set: default $T85nasdir)
# T85exifdestdir     -- Where the pics and movies are to be put into
# T85exiflogfile     -- The logfile of operation of this skript
# T85exifrenshdir    -- Location where the exifren shell skripts are
# T85exiffilemask    -- Which files to process
# T85exiftoolcall    -- How to call exiftool
# T85exiftoolopts    -- The options for the call to exiftool
# T85exifren_runfile -- Log starts of exifren, may be used by T85get.sh to detect T85exifren is running
#                    -- set to NUL if not wanted
# T85DBG             -- give debug output
#
# read: file T85exifren_debug in T85exifsrcdir existing: Set T85DBG
# write: file T85exifren_runfile in T85exifsrcdir does a protocol of run and may be detected by T85get.sh
#
# If parameters are given, the first three of these options get overridden

# if Parameter 1 is set, it is the T85exifsrcdir
[ "$1" ] && T85exifsrcdir="$1"
[ -z "$T85exifsrcdir" ] && T85exifsrcdir="$T85nasdir"
[ -z "$T85exifsrcdir" ] && T85exifsrcdir="."

# if Parameter 2 is set, it is the T85exifdestdir
[ "$2" ] && T85exifdestdir="$2"
[ -z "$T85exifdestdir" ] && T85exifdestdir="$T85exifsrcdir"

[ -z "$T85exiflogfile" ] && T85exiflogfile=$T85exifsrcdir/T85exifren.log
# if Parameter 3 is set, it is the logfile
[ "$3" ] && T85exiflogfile="$3"

[ -e "$T85exifsrcdir/T85exifren_debug" ] && T85DBG=1

# minimum age of picture Format YYYY:MM:DD, used for exiftool comparison
# so it is fine to give e. g. only year
[ -z "$T85exifrenmindate" ] && T85exifrenmindate="2020"

[ -z "$T85exiftoolcall" ] && T85exiftoolcall="exiftool"

[ -z "${T85exifren_runfile}" ] && T85exifren_runfile="T85exifren_runfile"

[ -z "$T85typename" ] && T85typename=( "PHOTO" "MOVIE" )
[ -z "$T85typeext"  ] && T85typeext=( "JPG" "MP4" )
[ -z "$T85typemask" ] && T85typemask=( '*.JPG' '*.MP4' )

if (( ${#T85typename[@]} != ${#T85typemask[@]} )); then
  echo Array types should have equal element count T85typename ${T85typename[@]} T85typemask ${T85typemask[@]}
fi

if (( ${#T85typename[@]} != ${#T85typeext[@]} )); then
  echo Array ext should have equal element count T85typename ${T85typename[@]} T85typeext ${T85typeext[@]}
fi

unset filesmask
for i in "${!T85typemask[@]}";
do
  filesmask="$filesmask $T85exifsrcdir/${T85typemask[$i]}"
done

now=$(date +"%Y%m%d_%H%M%S")
[[ "$T85DBG" ]] && echo "dbg $now filesmask is -$filesmask-"
[[ "$T85DBG" ]] && echo "dbg $now T85exifsrcdir is -$T85exifsrcdir-, T85destdir is -$T85exifdestdir-"

# options for exiftool:
# which extensions to care about (valid for all exiftool operations 
if [ -z "$T85exiftoolexts" ]; then
  T85exiftoolexts=""
  for i in "${!T85typeext[@]}";
  do
    T85exiftoolexts="$T85exiftoolexts -ext ${T85typeext[$i]}"
  done
fi
now=$(date +"%Y%m%d_%H%M%S")
[[ "$T85DBG" ]] && echo "dbg $now exiftoolexts are -$T85exiftoolexts-"

# options for exiftool:
# rename files with no tags to original file date/time 
[ -z "$T85exiftoolnotagopts" ] &&  T85exiftoolnotagopts="'-filename<filemodifydate' -d %Y%m%d%H%M%S_nodate_%%f.%%ue"
now=$(date +"%Y%m%d_%H%M%S")
[[ "$T85DBG" ]] && echo "dbg $now exiftoolnotagopts are $T85exiftoolnotagopts"

[ -z "$T85exiftoolnotagcond" ] &&  T85exiftoolnotagcond="-if 'not defined \$createdate or \$createdate eq \"\"'"
now=$(date +"%Y%m%d_%H%M%S")
[[ "$T85DBG" ]] && echo "dbg $now exiftoolnotagcond is $T85exiftoolnotagcond"

# options for exiftool:
# rename files older than T85exifrenmindate to contain the file modify date, not the createdate
# note: One \$ for this shell. One \\ for the call with bash -c where variable get substituted again
[ -z "$T85exiftoolageopts" ] && T85exiftoolageopts="'-filename<filemodifydate' -d %Y%m%d%H%M%S_wrongdate_%%f.%%ue"
now=$(date +"%Y%m%d_%H%M%S")
[[ "$T85DBG" ]] && echo "dbg $now exiftoolageopts are $T85exiftoolageopts"

[ -z "$T85exiftoolagecond" ] && T85exiftoolagecond="-if '\$createdate lt $T85exifrenmindate and \$filename !~ /^20[0-9]{6}/'"
[[ "$T85DBG" ]] && echo "dbg $now exiftoolagecond is $T85exiftoolagecond"

# options for exiftool:
# rename file to datetime_(originalfilename) like Hunting Cam Pro App
# set the file modification date to create date the meta data says
# exclude all files that start with "20" (only works this century, shame on me) as they seem to be renamed already
# deal only with jpg mp4 extension files
now=$(date +"%Y%m%d_%H%M%S")
[ -z "$T85exiftoolopts" ] && T85exiftoolopts="'-filename<createdate' '-filemodifydate<createdate' -d %Y%m%d%H%M%S_%%f.%%ue -if '\$filename !~ /^20[0-9]{6}/'"
[[ "$T85DBG" ]] && echo "dbg $now exiftoolopts are $T85exiftoolopts"

now=$(date +"%Y%m%d_%H%M%S")
[[ "$T85exifren_runfile" != "NUL" ]] && echo $now>>$T85exifsrcdir/$T85exifren_runfile 

# make ls show JPG also when asked for *.jpg and viceversa
shopt -s nocaseglob

# start now
[ -z "$T85exifrenshdir" ] && T85exifrenshdir="$(dirname "${BASH_SOURCE[0]}")"
cd "$T85exifrenshdir"
now=$(date +"%Y%m%d_%H%M%S")
echo "$now start T85exifren.sh in $PWD"

trap ctrl_c INT
function ctrl_c()
{
  now=$(date +"%Y%m%d_%H%M%S")
  [[ "$T85DBG" ]] && echo "dbg $now Ctrl-C trapped, remove progress"
  rm progress
}

# check if already running
if [ -f "progress" ]; then
  sleep 1
  now=$(date +"%Y%m%d_%H%M%S")
  if [ -f "progress" ]; then
    echo $now' already progress, exiting' | tee -a "$T85exiflogfile"
	exit 1
  else
    [[ "$T85DBG" ]] && echo "dbg $now check run ok, no other instance seems active"
  fi
else
  sleep 0.1
  if [ -f "progress" ]; then
    now=$(date +"%Y%m%d_%H%M%S")
    echo "$now found progress after it was not present!"
  fi
fi

# is there anything to do?
now=$(date +"%Y%m%d_%H%M%S")
unset dosomething
for i in "${!T85typemask[@]}";
do
  [[ "$T85DBG" ]] && echo "dbg find $T85exifsrcdir -maxdepth 1 -type f -iname \"${T85typemask[$i]}\""
  if [[ "$(eval find $T85exifsrcdir -maxdepth 1 -type f -iname \"${T85typemask[$i]}\")" ]];
  then
    echo $now" found something to care about in ${T85typemask[$i]}"
    dosomething=1
    break
  else
    [[ "$T85DBG" ]] && echo "dbg nothing found ${T85typemask[$i]}"
  fi
done

if [[ "$dosomething" ]];
then
  echo $now" found something to care about"
else
  echo $now" no file found (exit)"
  exit 2
fi

now=$(date +"%Y%m%d_%H%M%S")
echo $now' T85exifren.sh -- start processing' | tee -a "$T85exiflogfile"
# write semaphore for check if running
echo $now' running'>progress

# rename files
[[ "$T85DBG" ]] && echo "dbg before run exiftool"
ls -l $filesmask | tee -a "$T85exiflogfile"

# if the clock of the camera is not set, it will default to some special date, e.g. 2019-01-01
# files that are older than T85exifmindate will be renamed to filedate

[[ "$T85DBG" ]] && echo "dbg run exiftool with no tag opts to rename files without CreateDate"
[[ "$T85DBG" ]] && echo $T85exiftoolcall "$T85exiftoolnotagopts $T85exiftoolnotagcond $T85exiftoolexts $T85exifsrcdir/."
bash -c "$T85exiftoolcall $T85exiftoolnotagopts $T85exiftoolnotagcond $T85exiftoolexts $T85exifsrcdir/." | tee -a "$T85exiflogfile"

[[ "$T85DBG" ]] && echo "dbg run exiftool with min age opts to rename files with wrong CreateDate"
[[ "$T85DBG" ]] && echo $T85exiftoolcall "$T85exiftoolageopts $T85exiftoolagecond $T85exiftoolexts $T85exifsrcdir/."
bash -c "$T85exiftoolcall $T85exiftoolageopts $T85exiftoolagecond $T85exiftoolexts $T85exifsrcdir/." | tee -a "$T85exiflogfile"

[[ "$T85DBG" ]] && echo "dbg run exiftool to rename files"
[[ "$T85DBG" ]] && echo $T85exiftoolcall "$T85exiftoolageopts $T85exiftoolagecond $T85exiftoolexts $T85exifsrcdir/."
bash -c "$T85exiftoolcall $T85exiftoolopts $T85exiftoolexts $T85exifsrcdir/." | tee -a "$T85exiflogfile"

now=$(date +"%Y%m%d_%H%M%S")
[[ "$T85DBG" ]] && echo "dbg $now move files to destination into the wanted folders"
find "$T85exifsrcdir" -maxdepth 1 -type f -iname '20[0-9][0-9][0-9][0-9][0-9][0-9]*' -exec "$T85exifrenshdir/T85move.sh" {} $T85exifdestdir \; | tee -a "$T85exiflogfile"

sleep 1

# remove semaphore
rm progress

now=$(date +"%Y%m%d_%H%M%S")
echo $now' T85exifren.sh -- end' | tee -a "$T85exiflogfile"

