#!/bin/bash
#
# Move a file, delete original if existing and identical
# author timm@ganske.org 2020
# Parameter 1 filename, Parameter 2 destdir
#

if [[ $# -eq 0 ]]; then
  echo "Usage: $0 file destdir (subfoldername)"
echo "   moves the file, but only if it not already exists"
echo "    (if exists and is binary identical, file will be deleted)"
ehco "    (if subfoldername is not given, first 8 char of filename will be taken)"
  exit 0
fi

now=$(date +"%Y%m%d_%H%M%S")
filename="$(basename -- $1)"
foldername=$2/${filename:0:8}
[ "$3" ] && foldername=$3
echo $now' move '$1' (filename '$filename') to folder '$foldername
[ ! -d "$foldername" ] && mkdir -p "$foldername"

# check return code of mv, it will indicate if exists already
#   if so, then check if same time and size. If yes delete
#   if files are different, needs to be checked manually
mv -n $1 $foldername
# if [ $? -ne 0 ] ; then -- did not work!
if [ -f "$1" ] ; then
  echo "error $? on mv file $1 $foldername"
  if [ -f "$foldername/$filename" ] ; then
    if cmp -s "$1" "$foldername/$filename"; then
	  rm $1
	  echo "files were identical, deleted $1"
	else
	  echo "files $1 are different! Do nothing, please check manually"
	fi
  else
    echo dest for mv does not exist, but an error occured - no idea what to do, so do nothing
  fi
fi
